import os
import json
import logging
from datetime import datetime, timedelta
from time import sleep

import requests

logging.basicConfig(
    format='%(asctime)s %(levelname)s: %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
    level=logging.DEBUG
)

class SimpleScoreSaberAPI:
    BASE_URL = "https://scoresaber.com/api"

    def __init__(self, cache_expiry_days=1):
        self.session = requests.Session()
        self.cache_expiry_days = cache_expiry_days
        self.CACHE_DIR = self._determine_cache_dir()
        if not os.path.exists(self.CACHE_DIR):
            os.makedirs(self.CACHE_DIR)
            logging.info(f"Created cache directory: {self.CACHE_DIR}")

    def _determine_cache_dir(self):
        home_cache = os.path.expanduser("~/.cache")
        saberlist_cache = os.path.join(home_cache, "saberlist")
        scoresaber_cache = os.path.join(saberlist_cache, "scoresaber")
        
        if os.path.exists(home_cache):
            if not os.path.exists(saberlist_cache):
                try:
                    os.makedirs(saberlist_cache)
                    logging.info(f"Created cache directory: {saberlist_cache}")
                except OSError as e:
                    logging.warning(f"Failed to create {saberlist_cache}: {e}")
                    return os.path.join(os.getcwd(), ".cache")
            if not os.path.exists(scoresaber_cache):
                try:
                    os.makedirs(scoresaber_cache)
                    logging.info(f"Created cache directory: {scoresaber_cache}")
                except OSError as e:
                    logging.warning(f"Failed to create {scoresaber_cache}: {e}")
                    return os.path.join(os.getcwd(), ".cache")
            return scoresaber_cache
        else:
            logging.info("~/.cache doesn't exist, using local .cache directory")
            return os.path.join(os.getcwd(), ".cache", "scoresaber")

    def _get_cache_filename(self, min_star, max_star):
        return os.path.join(self.CACHE_DIR, f"ranked_maps_{min_star}_{max_star}.json")

    def _is_cache_valid(self, cache_file):
        if not os.path.exists(cache_file):
            return False
        file_modified_time = datetime.fromtimestamp(os.path.getmtime(cache_file))
        return datetime.now() - file_modified_time < timedelta(days=self.cache_expiry_days)

    def clear_cache(self, min_star=None, max_star=None):
        """
        Clear cached data.

        :param min_star: Minimum star rating filter for specific cache file
        :param max_star: Maximum star rating filter for specific cache file
        """
        if min_star is not None and max_star is not None:
            cache_file = self._get_cache_filename(min_star, max_star)
            if os.path.exists(cache_file):
                os.remove(cache_file)
                logging.debug(f"Cleared cache for ranked maps (stars: {min_star}-{max_star})")
        else:
            for file in os.listdir(self.CACHE_DIR):
                file_path = os.path.join(self.CACHE_DIR, file)
                if os.path.isfile(file_path):
                    os.remove(file_path)
            logging.debug("Cleared all ranked maps cache")

    def get_cache_dir(self):
        """
        Get the cache directory path.

        :return: Cache directory path as a string
        """
        return self.CACHE_DIR

    def get_ranked_maps(self, min_star=5, max_star=10, use_cache=True, limit=100, max_pages=None):
        """
        Retrieve all ranked maps within the specified star range.
        The api doesn't actually tell you how many results there are, so we have to fetch all pages until we get back no data

        :param min_star: Minimum star rating (inclusive)
        :param max_star: Maximum star rating (inclusive)
        :param use_cache: Whether to use cached data if available (default: True)
        :param max_pages: Maximum number of pages to fetch (default: None, fetch all)
        :return: List of ranked maps
        """
        cache_file = self._get_cache_filename(min_star, max_star)

        if use_cache and self._is_cache_valid(cache_file):
            logging.debug(f"Using cached data for ranked maps (stars: {min_star}-{max_star})")
            with open(cache_file, 'r') as f:
                return json.load(f)

        logging.debug(f"Fetching ranked maps from API (stars: {min_star}-{max_star})")
        url = f"{self.BASE_URL}/leaderboards"
        
        all_maps = []
        page = 1

        while max_pages is None or page <= max_pages:
            params = {
                "minStar": min_star,
                "maxStar": max_star,
                "unique": "true",
                "ranked": "true",
                "page": page
            }
            try:
                response = self.session.get(url, params=params)
                response.raise_for_status()
                data = response.json()

                leaderboards = data.get('leaderboards', [])
                if not leaderboards:
                    logging.debug("Empty page reached, stopping")
                    break

                all_maps.extend(leaderboards)

                logging.info(f"Fetched page {page} with {len(leaderboards)} maps.")

                page += 1
                sleep(0.5)

            except requests.exceptions.RequestException as e:
                logging.error(f"Error fetching ranked maps: {e}")
                break

        # Cache the results
        with open(cache_file, 'w') as f:
            json.dump(all_maps, f, indent=4)
            logging.debug(f"Cached {len(all_maps)} ranked maps to {cache_file}")

        return all_maps
