import argparse
import logging
import os
import sys
from dotenv import load_dotenv
load_dotenv()
LOG_LEVEL = os.environ.get('LOG_LEVEL', 'INFO').upper()
CACHE_EXPIRY_DAYS = int(os.environ.get('CACHE_EXPIRY_DAYS', 7))

import logging
logging.basicConfig(
    format='%(asctime)s %(levelname)s: %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
    level=LOG_LEVEL
)

from helpers.PlaylistBuilder import PlaylistBuilder
from helpers.ScoreSaberAPI import ScoreSaberAPI
from helpers.BeatLeaderAPI import BeatLeaderAPI
from helpers.SimpleBeatLeaderAPI import SimpleBeatLeaderAPI
from helpers.SimpleScoreSaberAPI import SimpleScoreSaberAPI
from helpers.SimpleBeatSaverAPI import SimpleBeatSaverAPI

from saberlist.utils import reset_history
from saberlist.playlist_strategies.beatleader import (
    playlist_strategy_beatleader_oldscores,
    playlist_strategy_beatleader_oldscores_stars,
    playlist_strategy_ranked_both,
)
from saberlist.playlist_strategies.scoresaber import (
    playlist_strategy_scoresaber_oldscores,
    playlist_strategy_scoresaber_ranked,
)
from saberlist.playlist_strategies.accuracy import (
    playlist_strategy_beatleader_accuracy_gaps,
    playlist_strategy_scoresaber_accuracy_gaps,
    playlist_strategy_beatleader_accuracy_gaps_star_range,
)
from saberlist.playlist_strategies.beatsaver import (
    playlist_strategy_beatsaver_curated,
    playlist_strategy_beatsaver_mappers,
)

def saberlist() -> None:
    """
    Generate a playlist of songs using a specified strategy.
    Avoids reusing the same song+difficulty in a playlist based on history.
    """
    args = parse_args_subcommands()
    strategy = args.subcommand

    # If the user requested a reset, do that before anything else
    if getattr(args, 'reset', False):
        reset_history(strategy)
        sys.exit(0)

    # Then call the strategy-based logic
    if strategy == 'scoresaber_oldscores':
        playlist_data, playlist_title = playlist_strategy_scoresaber_oldscores(
            ScoreSaberAPI(cache_expiry_days=CACHE_EXPIRY_DAYS)
        )
        playlist_builder = PlaylistBuilder()

    elif strategy == 'beatleader_oldscores_stars':
        stars = input("Enter star level (Default: 6.0): ") or 6.0
        playlist_data, playlist_title = playlist_strategy_beatleader_oldscores_stars(
            SimpleBeatLeaderAPI(cache_expiry_days=CACHE_EXPIRY_DAYS),
            stars=float(stars)
        )
        playlist_builder = PlaylistBuilder(covers_dir='./covers/pajamas')

    elif strategy == 'beatleader_oldscores':
        playlist_data, playlist_title = playlist_strategy_beatleader_oldscores(
            BeatLeaderAPI(cache_expiry_days=CACHE_EXPIRY_DAYS)
        )
        playlist_builder = PlaylistBuilder()

    elif strategy == 'scoresaber_ranked':
        playlist_data, playlist_title = playlist_strategy_scoresaber_ranked(
            SimpleScoreSaberAPI(cache_expiry_days=CACHE_EXPIRY_DAYS)
        )
        playlist_builder = PlaylistBuilder(covers_dir='./covers/scoresaber')

    elif strategy == 'ranked_both':
        playlist_data, playlist_title = playlist_strategy_ranked_both(
            SimpleBeatLeaderAPI(cache_expiry_days=CACHE_EXPIRY_DAYS),
            SimpleScoreSaberAPI(cache_expiry_days=CACHE_EXPIRY_DAYS)
        )
        playlist_builder = PlaylistBuilder(covers_dir='./covers/kaiju')

    elif strategy == 'beatleader_accuracy_gaps':
        playlist_data, playlist_title = playlist_strategy_beatleader_accuracy_gaps(
            SimpleBeatLeaderAPI(cache_expiry_days=CACHE_EXPIRY_DAYS)
        )
        playlist_builder = PlaylistBuilder(covers_dir='./covers/pajamas')

    elif strategy == 'beatleader_accuracy_gaps_star_range':
        playlist_data, playlist_title = playlist_strategy_beatleader_accuracy_gaps_star_range(
            SimpleBeatLeaderAPI(cache_expiry_days=CACHE_EXPIRY_DAYS), 
            star_level=args.star_level
        )
        playlist_builder = PlaylistBuilder(covers_dir='./covers/pajamas')

    elif strategy == 'scoresaber_accuracy_gaps':
        playlist_data, playlist_title = playlist_strategy_scoresaber_accuracy_gaps(
            ScoreSaberAPI(cache_expiry_days=CACHE_EXPIRY_DAYS)
        )
        playlist_builder = PlaylistBuilder(covers_dir='./covers/scoresaber')

    elif strategy == 'beatsaver_curated':
        playlist_data, playlist_title = playlist_strategy_beatsaver_curated(SimpleBeatSaverAPI())
        playlist_builder = PlaylistBuilder(covers_dir='./covers/curated')

    elif strategy == 'beatsaver_mappers':
        playlist_data, playlist_title = playlist_strategy_beatsaver_mappers(SimpleBeatSaverAPI())
        playlist_builder = PlaylistBuilder(covers_dir='./covers/pajamas')

    elif strategy == 'blank_playlist':
        playlist_data = []
        playlist_title = input("Enter playlist title: ")
        playlist_builder = PlaylistBuilder(covers_dir='./covers/pajamas')

    else:
        logging.error(f"Unknown strategy '{strategy}'")
        return

    if not playlist_data and strategy != 'blank_playlist':
        logging.info("No new scores found to add to the playlist.")
        return

    playlist_builder.create_playlist(
        playlist_data,
        playlist_title=playlist_title,
        playlist_author="SaberList Tool"
    )

def parse_args_subcommands():
    """
    Parse sub-commands for each strategy instead of using a single `-s/--strategy`.
    """
    parser = argparse.ArgumentParser(
        description="Generate Beat Saber playlists"
    )

    subparsers = parser.add_subparsers(
        title="Available Strategies",
        dest="subcommand",
        help="Choose which sub-command (strategy) to run"
    )

    # -------- scoresaber_oldscores --------
    parser_ss_old = subparsers.add_parser("scoresaber_oldscores",
                                          help="Generate a playlist using ScoreSaber old-scores strategy")
    parser_ss_old.add_argument("-r", "--reset",
                               action="store_true",
                               help="Reset the history for scoresaber_oldscores")

    # -------- beatleader_oldscores_stars --------
    parser_bl_old_stars = subparsers.add_parser("beatleader_oldscores_stars",
                                                  help="Generate a playlist using BeatLeader old-scores-stars strategy")
    parser_bl_old_stars.add_argument("-r", "--reset",
                                     action="store_true",
                                     help="Reset the history for beatleader_oldscores_stars")

    # -------- beatleader_oldscores --------
    parser_bl_old = subparsers.add_parser("beatleader_oldscores",
                                          help="Generate a playlist using BeatLeader old-scores strategy")
    parser_bl_old.add_argument("-r", "--reset",
                               action="store_true",
                               help="Reset the history for beatleader_oldscores")

    # -------- scoresaber_ranked --------
    parser_ss_ranked = subparsers.add_parser("scoresaber_ranked",
                                              help="Generate a playlist using scoresaber_ranked strategy")
    parser_ss_ranked.add_argument("-r", "--reset",
                                   action="store_true",
                                   help="Reset the history for scoresaber_ranked")

    # -------- ranked_both --------
    parser_ranked_both = subparsers.add_parser("ranked_both",
                                              help="Generate a playlist using ranked_both strategy")
    parser_ranked_both.add_argument("-r", "--reset",
                                   action="store_true",
                                   help="Reset the history for ranked_both")

    # -------- beatleader_accuracy_gaps --------
    parser_bl_acc_gaps = subparsers.add_parser("beatleader_accuracy_gaps",
                                               help="Generate a playlist using BeatLeader accuracy gaps strategy")
    parser_bl_acc_gaps.add_argument("-r", "--reset",
                                    action="store_true",
                                    help="Reset the history for beatleader_accuracy_gaps")

    # -------- beatleader_accuracy_gaps_star_range --------
    parser_bl_acc_stars = subparsers.add_parser("beatleader_accuracy_gaps_star_range",
                                                help="Generate a playlist for accuracy gaps within a star range (BeatLeader)")
    parser_bl_acc_stars.add_argument("-r", "--reset",
                                     action="store_true",
                                     help="Reset the history for beatleader_accuracy_gaps_star_range")
    parser_bl_acc_stars.add_argument("--star-level",
                                     type=float,
                                     help="Star level to filter on")

    # -------- scoresaber_accuracy_gaps --------
    parser_ss_acc_gaps = subparsers.add_parser("scoresaber_accuracy_gaps",
                                               help="Generate a playlist using ScoreSaber accuracy gap strategy")
    parser_ss_acc_gaps.add_argument("-r", "--reset",
                                    action="store_true",
                                    help="Reset the history for scoresaber_accuracy_gaps")

    # -------- beatsaver_curated --------
    parser_bs_curated = subparsers.add_parser("beatsaver_curated",
                                              help="Generate a curated BeatSaver playlist")
    parser_bs_curated.add_argument("-r", "--reset",
                                   action="store_true",
                                   help="Reset the history for beatsaver_curated")

    # -------- beatsaver_mappers --------
    parser_bs_mappers = subparsers.add_parser("beatsaver_mappers",
                                              help="Generate a playlist for specified BeatSaver mappers")
    parser_bs_mappers.add_argument("-r", "--reset",
                                   action="store_true",
                                   help="Reset the history for beatsaver_mappers")

    # -------- blank_playlist --------
    parser_blank = subparsers.add_parser("blank_playlist",
                                         help="Generate a blank playlist (no songs, just a descriptor)")
    parser_blank.add_argument("-r", "--reset",
                              action="store_true",
                              help="Reset the history for blank_playlist (usually unnecessary)")

    # If no arguments passed, print help
    if len(sys.argv) == 1:
        parser.print_help(sys.stderr)
        sys.exit(1)

    return parser.parse_args()
